@@ -22,6 +22,7 @@ import logging
 import os
 import sys
 import threading
+import warnings
 from typing import Any, List, Callable, Hashable, NamedTuple, Iterator, Optional
 
 from jax._src import lib
@@ -78,13 +79,10 @@ class Config:
     self._update_hooks = {}
 
   def update(self, name, val):
-    if self.use_absl:
-      setattr(self.absl_flags.FLAGS, name, val)
-    else:
-      self.check_exists(name)
-      if name not in self.values:
-        raise Exception(f"Unrecognized config option: {name}")
-      self.values[name] = val
+    self.check_exists(name)
+    if name not in self.values:
+      raise Exception(f"Unrecognized config option: {name}")
+    self.values[name] = val
 
     hook = self._update_hooks.get(name, None)
     if hook:
@@ -98,11 +96,10 @@ class Config:
     return self._read(name)
 
   def _read(self, name):
-    if self.use_absl:
-      return getattr(self.absl_flags.FLAGS, name)
-    else:
-      self.check_exists(name)
+    try:
       return self.values[name]
+    except KeyError:
+      raise AttributeError(f"Unrecognized config option: {name}")
 
   def add_option(self, name, default, opt_type, meta_args, meta_kwargs,
                  update_hook=None):
@@ -159,7 +156,9 @@ class Config:
 
   def complete_absl_config(self, absl_flags):
     for name, _ in self.values.items():
-      self.update(name, getattr(absl_flags.FLAGS, name))
+      flag = absl_flags.FLAGS[name]
+      if flag.present:
+        self.update(name, flag.value)
 
   def parse_flags_with_absl(self):
     global already_configured_with_absl
@@ -242,7 +241,7 @@ class Config:
     self._contextmanager_flags.add(name)
 
     def get_state(self):
-      val = getattr(_thread_local_state, name, unset)
+      val = _thread_local_state.__dict__.get(name, unset)
       return val if val is not unset else self._read(name)
     setattr(Config, name, property(get_state))
 
@@ -279,7 +278,7 @@ class Config:
     self._contextmanager_flags.add(name)
 
     def get_state(self):
-      val = getattr(_thread_local_state, name, unset)
+      val = _thread_local_state.__dict__.get(name, unset)
       return val if val is not unset else self._read(name)
     setattr(Config, name, property(get_state))
 
@@ -321,7 +320,7 @@ class Config:
     self._contextmanager_flags.add(name)
 
     def get_state(self):
-      val = getattr(_thread_local_state, name, unset)
+      val = _thread_local_state.__dict__.get(name, unset)
       return val if val is not unset else self._read(name)
     setattr(Config, name, property(get_state))
 
@@ -362,7 +361,7 @@ class Config:
     self._contextmanager_flags.add(name)
 
     def get_state(self):
-      val = getattr(_thread_local_state, name, unset)
+      val = _thread_local_state.__dict__.get(name, unset)
       return val if val is not unset else self._read(name)
     setattr(Config, name, property(get_state))
 
@@ -448,7 +447,7 @@ class Config:
     self._contextmanager_flags.add(name)
 
     def get_state(self):
-      val = getattr(_thread_local_state, name, unset)
+      val = _thread_local_state.__dict__.get(name, unset)
       return val if val is not unset else self._read(name)
     setattr(Config, name, property(get_state))
 
@@ -470,7 +469,7 @@ class Config:
     return (axis_env_state, self.x64_enabled, self.jax_numpy_rank_promotion,
             self.jax_default_matmul_precision, self.jax_dynamic_shapes,
             self.jax_numpy_dtype_promotion, self.jax_default_device,
-            self.jax_array)
+            self.jax_array, self.jax_threefry_partitionable)
 
 class NoDefault: pass
 no_default = NoDefault()
@@ -488,7 +487,7 @@ class _StateContextManager:
     self._default_value = default_value
 
   @contextlib.contextmanager
-  def __call__(self, new_val=no_default):
+  def __call__(self, new_val: Any = no_default):
     if new_val is no_default:
       if self._default_value is not no_default:
         new_val = self._default_value  # default_value provided to constructor
@@ -559,6 +558,7 @@ class _GlobalExtraJitContext(NamedTuple):
   numpy_dtype_promotion: Optional[str] = None
   default_matmul_precision: Optional[Any] = None
   dynamic_shapes: bool = False
+  threefry_partitionable: bool = False
 
 
 def _update_global_jit_state(**kw):
@@ -680,6 +680,16 @@ jax2tf_default_experimental_native_lowering = config.define_bool_state(
     )
 )
 
+jax2tf_use_stablehlo = config.define_bool_state(
+    name='jax2tf_use_stablehlo',
+    default=bool_env('JAX2TF_USE_STABLEHLO', True),
+    help=(
+        'DO NOT USE, highly experimental. Use in conjunction with jax2tf '
+        'experimental_native_lowering, to use StableHLO instead of MHLO as '
+        'the serialization format.'
+    )
+)
+
 jax_platforms = config.define_string_state(
     name='jax_platforms',
     default=None,
@@ -741,14 +751,26 @@ parallel_functions_output_gda = config.define_bool_state(
     help='If True, pjit will output GDAs.')
 
 def _update_jax_array_global(val):
+  if not val:
+    warnings.warn(
+        'DeviceArray, ShardedDeviceArray, and GlobalDeviceArray have been '
+        'deprecated. Please use `jax.Array`. See '
+        'https://jax.readthedocs.io/en/latest/jax_array_migration.html on how '
+        'to migrate to `jax.Array`.', DeprecationWarning)
   lib.jax_jit.global_state().jax_array = val
 
 def _update_jax_array_thread_local(val):
+  if not val:
+    warnings.warn(
+        'DeviceArray, ShardedDeviceArray, and GlobalDeviceArray have been '
+        'deprecated. Please use `jax.Array`. See '
+        'https://jax.readthedocs.io/en/latest/jax_array_migration.html on how '
+        'to migrate to `jax.Array`.', DeprecationWarning)
   lib.jax_jit.thread_local_state().jax_array = val
 
 jax_array = config.define_bool_state(
     name='jax_array',
-    default=False,
+    default=True,
     upgrade=True,
     update_global_hook = _update_jax_array_global,
     update_thread_local_hook = _update_jax_array_thread_local,
@@ -805,7 +827,11 @@ threefry_partitionable = config.define_bool_state(
           'standard jax.random pseudo-random number generation may result '
           'in extraneous communication and/or redundant distributed '
           'computation. With this flag, the communication overheads disappear '
-          'in some cases.'))
+          'in some cases.'),
+    update_global_hook=lambda val: _update_global_jit_state(
+        threefry_partitionable=val),
+    update_thread_local_hook=lambda val: update_thread_local_jit_state(
+        threefry_partitionable=val))
 
 enable_custom_vjp_by_custom_transpose = config.define_bool_state(
     name='jax_enable_custom_vjp_by_custom_transpose',
@@ -984,9 +1010,10 @@ traceback_filtering = config.define_enum_state(
 
 # This flag is for internal use.
 # TODO(tianjianlu): Removes once we always enable cusparse lowering.
+# TODO(b/262050896): Set to true after bug is fixed
 bcoo_cusparse_lowering = config.define_bool_state(
     name='jax_bcoo_cusparse_lowering',
-    default=True,
+    default=False,
     help=('Enables lowering BCOO ops to cuSparse.'))
 
 # TODO(mattjj): remove this flag when we ensure we only succeed at trace-staging
@@ -1001,13 +1028,6 @@ config.define_bool_state(
     update_thread_local_hook=lambda val: \
       update_thread_local_jit_state(dynamic_shapes=val))
 
-# This flag is temporary during rollout of the remat barrier.
-# TODO(parkers): Remove if there are no complaints.
-config.define_bool_state(
-    name='jax_remat_opt_barrier',
-    default=(lib.version >= (0, 3, 6)),
-    help=('Enables using optimization-barrier op for lowering remat.'))
-
 # TODO(b/205307544): Remove flag once coordination service has rolled out.
 config.define_bool_state(
     name='jax_coordination_service',
