@@ -21,9 +21,8 @@ from functools import partial
 import itertools
 import time
 from typing import (
-    Any, Callable, Dict, Iterable, Iterator, Optional, Sequence,
-    Set, Tuple, List, Type, Union)
-from typing_extensions import Protocol
+    Any, Callable, Dict, Iterable, Iterator, Optional, Protocol,
+    Sequence, Set, Tuple, List, Type, Union)
 import logging
 import os
 import re
@@ -36,16 +35,21 @@ import jax
 from jax import core
 from jax import linear_util as lu
 from jax.errors import UnexpectedTracerError
+from jax.monitoring import record_event_duration_secs
 import jax.interpreters.ad as ad
 import jax.interpreters.batching as batching
 import jax.interpreters.mlir as mlir
 import jax.interpreters.xla as xla
+from jax.interpreters import pxla
 import jax.interpreters.partial_eval as pe
+from jax._src import array
 from jax._src import device_array
 from jax._src import dtypes
 from jax._src import profiler
 from jax._src import stages
 from jax._src import traceback_util
+from jax._src.sharding import (PmapSharding, SingleDeviceSharding,
+                               OpShardingSharding, Sharding)
 from jax._src.abstract_arrays import array_types
 from jax._src.config import config, flags
 from jax._src.lib.mlir import ir
@@ -56,6 +60,8 @@ import jax._src.util as util
 from jax._src.util import flatten, unflatten
 from jax._src import path
 
+JAXPR_TRACE_EVENT = "/jax/core/compile/jaxpr_trace_duration"
+BACKEND_COMPILE_EVENT = "/jax/core/compile/backend_compile_duration"
 
 FLAGS = flags.FLAGS
 
@@ -92,7 +98,6 @@ _on_exit = False
 ArgSpec = Tuple[core.AbstractValue, Optional[Device]]
 
 def arg_spec(x: Any) -> ArgSpec:
-  from jax._src.sharding import PmapSharding
   from jax.experimental import pjit
 
   aval = xla.abstractify(x)
@@ -304,9 +309,6 @@ def not_none_device_or_backend_on_jit(backend, device, num_ins):
   """
   # TODO(yashkatariya): Remove this entire function when backend and device are
   # removed as arguments on jit.
-
-  from jax._src import sharding
-
   if device is not None and backend is not None:
     raise ValueError("can't specify both a device and a backend for jit, "
                      "got device={} and backend={}".format(device, backend))
@@ -320,7 +322,7 @@ def not_none_device_or_backend_on_jit(backend, device, num_ins):
   assert len(da) == 1
   # in_shardings will be marked as replicated regardless of whatever the input
   # had. Given that only a single device is allowed above, this is correct.
-  in_shardings = [sharding.OpShardingSharding.get_replicated(da)] * num_ins
+  in_shardings = [OpShardingSharding.get_replicated(da)] * num_ins
   return da, in_shardings
 
 
@@ -363,14 +365,13 @@ xla_callable = lu.cache(_xla_callable_uncached)
 
 
 def is_single_device_sharding(sharding) -> bool:
-  from jax._src.sharding import PmapSharding
   # Special case PmapSharding here because PmapSharding maps away an axis
   # and needs to be handled separately.test_pjit_single_device_sharding_add
   return len(sharding.device_set) == 1 and not isinstance(sharding, PmapSharding)
 
 
 @contextlib.contextmanager
-def log_elapsed_time(fmt: str):
+def log_elapsed_time(fmt: str, event: Optional[str] = None):
   if _on_exit:
     yield
   else:
@@ -379,6 +380,8 @@ def log_elapsed_time(fmt: str):
     yield
     elapsed_time = time.time() - start_time
     logger.log(log_priority, fmt.format(elapsed_time=elapsed_time))
+    if event is not None:
+      record_event_duration_secs(event, elapsed_time)
 
 
 def should_tuple_args(num_args: int, platform: str):
@@ -441,7 +444,8 @@ def lower_xla_callable(
     abstract_args = tuple(aval for aval, _ in fun.in_type)
 
   with log_elapsed_time(f"Finished tracing + transforming {fun.__name__} "
-                        "for jit in {elapsed_time} sec"):
+                        "for jit in {elapsed_time} sec",
+                        event=JAXPR_TRACE_EVENT):
     jaxpr, out_type, consts = pe.trace_to_jaxpr_final2(
         fun, pe.debug_info_final(fun, "jit"))
   out_avals, kept_outputs = util.unzip2(out_type)
@@ -639,7 +643,7 @@ def _xla_callable_device(nreps, backend, device,
   if nreps > 1:
     if device is not None or backend is not None:
       raise ValueError(f"can't specify device or backend for jit-of-pmap, "
-                       f"got device={device} and backend={backend}")
+                       f"got {device=} and {backend=}")
     return None
   else:
     # TODO(skye): dedup with C++ jit logic for determining jit device?
@@ -770,10 +774,8 @@ class SimpleResultHandler:
 
 def maybe_create_array_from_da(buf, aval, device):
   if config.jax_array:
-    from jax._src.array import ArrayImpl
-    from jax._src.sharding import SingleDeviceSharding
-    return ArrayImpl(aval, SingleDeviceSharding(buf.device()), [buf],
-                     committed=(device is not None), _skip_checks=True)
+    return array.ArrayImpl(aval, SingleDeviceSharding(buf.device()), [buf],
+                           committed=(device is not None), _skip_checks=True)
   else:
     return device_array.make_device_array(aval, device, buf)
 
@@ -1107,13 +1109,13 @@ def _cache_write(serialized_computation: Union[str, bytes, ir.Module],
   min_compile_time = config.jax_persistent_cache_min_compile_time_secs
   if min_compile_time:
     if compile_time_secs < min_compile_time:
-      logging.info(
+      logger.info(
           "Not writing persistent cache entry for '%s' because it took < %.2f "
           "seconds to compile (%.2fs)", module_name, min_compile_time,
           compile_time_secs)
       return
     else:
-      logging.info(
+      logger.info(
           "'%s' took at least %.2f seconds to compile (%.2fs), writing "
           "persistent cache entry", module_name, min_compile_time,
           compile_time_secs)
@@ -1190,7 +1192,8 @@ class XlaCompiledComputation(stages.XlaExecutable):
         device_assignment=(sticky_device,) if sticky_device else None)
     options.parameter_is_tupled_arguments = tuple_args
     with log_elapsed_time(f"Finished XLA compilation of {name} "
-                          "in {elapsed_time} sec"):
+                          "in {elapsed_time} sec",
+                          event=BACKEND_COMPILE_EVENT):
       compiled = compile_or_get_cached(backend, xla_computation, options,
                                        host_callbacks)
     buffer_counts = get_buffer_counts(out_avals, ordered_effects,
@@ -1251,6 +1254,12 @@ def check_arg_avals_for_call(ref_avals, arg_avals):
         f"called with:\n {arg_aval}")
 
 
+def _set_aval(val):
+  if val.aval is None:
+    val.aval = core.ShapedArray(val.shape, val.dtype)
+  return val
+
+
 def device_put(x, device: Optional[Device] = None) -> Tuple[Any, ...]:
   x = xla.canonicalize_dtype(x)
   try:
@@ -1289,6 +1298,19 @@ def _device_put_device_array(x: Union[device_array.DeviceArrayProtocol, device_a
   return (x.device_buffer,)
 for t in device_array.device_array_types:
   device_put_handlers[t] = _device_put_device_array
+
+def _device_put_jax_array(x, device: Optional[Device]):
+  if is_single_device_sharding(x.sharding):
+    x = _copy_device_array_to_device(_set_aval(x._arrays[0]), device)
+    return (x,)
+  else:
+    # Round trip via host if x is sharded. SDA also does a round trip via host.
+    return _device_put_array(x._value, device)
+device_put_handlers[array.ArrayImpl] = _device_put_jax_array
+
+device_put_handlers[pxla._ShardedDeviceArray] = _device_put_array
+device_put_handlers[pxla.pmap_lib.ShardedDeviceArray] = _device_put_array
+
 device_put_handlers[core.DArray] = lambda x, d: device_put(x._data, d)
 
 def _copy_device_array_to_device(
@@ -1318,8 +1340,6 @@ def _copy_device_array_to_device(
 
 def _copy_array_to_device(x: jax.Array, device: Optional[xc.Device]) -> jax.Array:
   """Copies `Array`s with SingleDeviceSharding to a different device."""
-  from jax._src import array, sharding
-
   if device is None:
     # no copying to be done because there's no target specified
     return x
@@ -1341,13 +1361,12 @@ def _copy_array_to_device(x: jax.Array, device: Optional[xc.Device]) -> jax.Arra
     backend = xb.get_device_backend(device)
     moved_buf = backend.buffer_from_pyval(np.asarray(buf), device)
   return array.ArrayImpl(
-      x.aval, sharding.SingleDeviceSharding(moved_buf.device()), [moved_buf],
+      x.aval, SingleDeviceSharding(moved_buf.device()), [moved_buf],
       committed=(device is not None))
 
 
 def _device_put_impl(
     x, device: Optional[Union[Device, jax.sharding.Sharding]] = None):
-  from jax._src import array, sharding
   from jax.interpreters import pxla
 
   try:
@@ -1356,7 +1375,7 @@ def _device_put_impl(
     raise TypeError(
         f"Argument '{x}' of type {type(x)} is not a valid JAX type") from err
 
-  if isinstance(device, sharding.Sharding):
+  if isinstance(device, Sharding):
     if not jax.config.jax_array:
       raise RuntimeError(
           "Please enable `jax_array` to use device_put with a `Sharding`. "
@@ -1367,7 +1386,7 @@ def _device_put_impl(
     if not s.is_fully_addressable:  # type: ignore
       raise ValueError(
           "device_put's second argument must be a Device or a Sharding which "
-          f"represents addressable devices, but got {sharding}")
+          f"represents addressable devices, but got {s}")
     if getattr(x, 'sharding', None) == s:
       return x
     # TODO(mattjj,yashkatariya,phawkins): more runtime fast resharding here?
