@@ -84,21 +84,21 @@ def dynamic_slice(operand: Array, start_indices: Union[Array, Sequence[ArrayLike
 
     >>> x = jnp.arange(12).reshape(3, 4)
     >>> x
-    DeviceArray([[ 0,  1,  2,  3],
-                 [ 4,  5,  6,  7],
-                 [ 8,  9, 10, 11]], dtype=int32)
+    Array([[ 0,  1,  2,  3],
+           [ 4,  5,  6,  7],
+           [ 8,  9, 10, 11]], dtype=int32)
 
     >>> dynamic_slice(x, (1, 1), (2, 3))
-    DeviceArray([[ 5,  6,  7],
-                 [ 9, 10, 11]], dtype=int32)
+    Array([[ 5,  6,  7],
+           [ 9, 10, 11]], dtype=int32)
 
     Note the potentially surprising behavior for the case where the requested slice
     overruns the bounds of the array; in this case the start index is adjusted to
     return a slice of the requested size:
 
     >>> dynamic_slice(x, (1, 1), (2, 4))
-    DeviceArray([[ 4,  5,  6,  7],
-                 [ 8,  9, 10, 11]], dtype=int32)
+    Array([[ 4,  5,  6,  7],
+           [ 8,  9, 10, 11]], dtype=int32)
   """
   start_indices = _dynamic_slice_indices(operand, start_indices)
   if jax.config.jax_dynamic_shapes:
@@ -129,25 +129,25 @@ def dynamic_update_slice(operand: Array, update: ArrayLike,
     >>> x = jnp.zeros(6)
     >>> y = jnp.ones(3)
     >>> dynamic_update_slice(x, y, (2,))
-    DeviceArray([0., 0., 1., 1., 1., 0.], dtype=float32)
+    Array([0., 0., 1., 1., 1., 0.], dtype=float32)
 
     If the update slice is too large to fit in the array, the start
     index will be adjusted to make it fit
 
     >>> dynamic_update_slice(x, y, (3,))
-    DeviceArray([0., 0., 0., 1., 1., 1.], dtype=float32)
+    Array([0., 0., 0., 1., 1., 1.], dtype=float32)
     >>> dynamic_update_slice(x, y, (5,))
-    DeviceArray([0., 0., 0., 1., 1., 1.], dtype=float32)
+    Array([0., 0., 0., 1., 1., 1.], dtype=float32)
 
     Here is an example of a two-dimensional slice update:
 
     >>> x = jnp.zeros((4, 4))
     >>> y = jnp.ones((2, 2))
     >>> dynamic_update_slice(x, y, (1, 2))
-    DeviceArray([[0., 0., 0., 0.],
-                 [0., 0., 1., 1.],
-                 [0., 0., 1., 1.],
-                 [0., 0., 0., 0.]], dtype=float32)
+    Array([[0., 0., 0., 0.],
+           [0., 0., 1., 1.],
+           [0., 0., 1., 1.],
+           [0., 0., 0., 0.]], dtype=float32)
   """
   start_indices = _dynamic_slice_indices(operand, start_indices)
   return dynamic_update_slice_p.bind(operand, update, *start_indices)
@@ -782,8 +782,7 @@ def _slice_transpose_rule(t, operand, *, start_indices, limit_indices, strides):
     pads = zip(start_indices, np.subtract(operand_shape, real_limits),
                np.subtract(strides, 1))
   result = lax.pad(t, lax._const(t, 0), pads)
-  assert result.shape == operand_shape, (
-    f"result.shape={result.shape} operand_shape={operand_shape}")
+  assert result.shape == operand_shape, f"{result.shape=} {operand_shape=}"
   return [result]
 
 
@@ -814,13 +813,8 @@ batching.primitive_batchers[slice_p] = _slice_batching_rule
 def _slice_lower(ctx, x, *, start_indices, limit_indices, strides):
   strides = strides or [1] * len(start_indices)
   aval_out, = ctx.avals_out
-  if core.is_opaque_dtype(aval_out.dtype):
-    return aval_out.dtype._rules.slice_mlir(
-        ctx, x, start_indices, limit_indices, strides)
-  return mhlo.SliceOp(x,
-                      mlir.dense_int_elements(start_indices),
-                      mlir.dense_int_elements(limit_indices),
-                      mlir.dense_int_elements(strides)).results
+  return [mlir.slice_op(ctx, x, aval_out,
+                        start_indices=start_indices, limit_indices=limit_indices, strides=strides)]
 
 mlir.register_lowering(slice_p, _slice_lower)
 
@@ -954,20 +948,10 @@ def _dynamic_slice_lower(ctx, x, *starts_and_dyn_sizes, slice_sizes):
   x_aval, *_ = ctx.avals_in
   start_indices, dyn = util.split_list(starts_and_dyn_sizes, [x_aval.ndim])
   aval_out, = ctx.avals_out
-  if core.is_opaque_dtype(aval_out.dtype) and dyn: raise NotImplementedError
-  if not dyn:
-    if core.is_opaque_dtype(aval_out.dtype):
-      return aval_out.dtype._rules.dynamic_slice_mlir(ctx, x, start_indices,
-                                                      slice_sizes)
-    return mhlo.DynamicSliceOp(x, start_indices,
-                               mlir.dense_int_elements(slice_sizes)).results
-  slice_sizes = lax._merge_dyn_shape(slice_sizes, dyn)
-  return mhlo.RealDynamicSliceOp(
-      mlir.aval_to_ir_type(aval_out), x,
-      mlir.shape_tensor(start_indices),
-      mlir.shape_tensor(slice_sizes),
-      mlir.shape_tensor([1] * len(slice_sizes))
-  ).results
+  if dyn:
+    aval_out = aval_out.update(shape=lax._merge_dyn_shape(slice_sizes, dyn))
+  return [mlir.dynamic_slice(ctx, aval_out, x, start_indices=start_indices)]
+
 mlir.register_lowering(dynamic_slice_p, _dynamic_slice_lower)
 
 # def _getslice_lower(ctx, x, lo, hi):
@@ -1069,11 +1053,8 @@ batching.primitive_batchers[dynamic_update_slice_p] = \
 
 def _dynamic_update_slice_lower(ctx, x, update, *start_indices):
   aval_out, = ctx.avals_out
-  if core.is_opaque_dtype(aval_out.dtype):
-    return aval_out.dtype._rules.dynamic_update_slice_mlir(
-        ctx, x, update, *start_indices)
-  return mhlo.DynamicUpdateSliceOp(mlir.aval_to_ir_type(aval_out), x, update,
-                                   start_indices).results
+  return [mlir.dynamic_update_slice(ctx, aval_out, x, update,
+                                    start_indices=start_indices)]
 
 mlir.register_lowering(dynamic_update_slice_p, _dynamic_update_slice_lower)
 
@@ -1171,7 +1152,7 @@ def _gather_shape_rule(operand, indices, *, dimension_numbers,
   if len(start_index_map) != indices.shape[index_vector_dim]:
     raise TypeError(f"Gather op has {len(start_index_map)} elements in "
                     f"start_index_map and the bound of dimension "
-                    f"index_vector_dim={index_vector_dim} of indices is "
+                    f"{index_vector_dim=} of indices is "
                     f"{indices.shape[index_vector_dim]}. These two "
                     f"numbers must be equal.")
 
@@ -1397,10 +1378,10 @@ def _gather_lower(ctx, operand, indices, *,
                   indices_are_sorted, mode, fill_value):
   aval_out, = ctx.avals_out
   if core.is_opaque_dtype(aval_out.dtype):
-    return aval_out.dtype._rules.gather_mlir(
-        ctx, operand, indices, dimension_numbers=dimension_numbers,
+    return [aval_out.dtype._rules.gather_mlir(
+        ctx, ctx.avals_in, aval_out, operand, indices, dimension_numbers=dimension_numbers,
         slice_sizes=slice_sizes, unique_indices=unique_indices,
-        indices_are_sorted=indices_are_sorted, mode=mode, fill_value=fill_value)
+        indices_are_sorted=indices_are_sorted, mode=mode, fill_value=fill_value)]
 
   if mode == GatherScatterMode.FILL_OR_DROP:
     gather_fill_fn = mlir.lower_fun(_gather_fill, multiple_results=False)
@@ -1417,12 +1398,28 @@ def _gather_lower(ctx, operand, indices, *,
     index_vector_dim=len(ctx.avals_in[1].shape) - 1,
     offset_dims=list(dimension_numbers.offset_dims),
     start_index_map=list(dimension_numbers.start_index_map))
-  return mhlo.GatherOp(
-      operand,
-      indices,
-      dnums,
-      mlir.dense_int_elements(slice_sizes),
-      indices_are_sorted=ir.BoolAttr.get(indices_are_sorted)).results
+  if not core.is_constant_shape(slice_sizes):
+    slice_sizes = mlir.eval_dynamic_shape(ctx, slice_sizes)
+    # TODO(burmako): Fix overly conservative type inference of DynamicGatherOp.
+    # For now use the build_generic so that we can specify the result type.
+    # return mhlo.DynamicGatherOp(
+    #     operand, indices, mlir.shape_tensor(slice_sizes),
+    #     dnums, indices_are_sorted=ir.BoolAttr.get(indices_are_sorted)).results
+    results = [mlir.aval_to_ir_type(aval_out)]
+    operands = [operand, indices, mlir.shape_tensor(slice_sizes)]
+    attributes = {
+        "dimension_numbers": dnums,
+        "indices_are_sorted": ir.BoolAttr.get(indices_are_sorted)
+    }
+    return mhlo.DynamicGatherOp.build_generic(
+        results=results, operands=operands, attributes=attributes).results
+  else:
+    return mhlo.GatherOp(
+        operand,
+        indices,
+        dnums,
+        mlir.dense_int_elements(slice_sizes),
+        indices_are_sorted=ir.BoolAttr.get(indices_are_sorted)).results
 
 mlir.register_lowering(gather_p, _gather_lower)
 
@@ -1507,7 +1504,7 @@ def _scatter_shape_rule(operand, indices, updates, *, update_jaxpr,
       indices.shape[index_vector_dim]):
     raise TypeError(f"Scatter op has {len(scatter_dims_to_operand_dims)} "
                     f"elements in scatter_dims_to_operand_dims and the bound "
-                    f"of dimension index_vector_dim={index_vector_dim} of "
+                    f"of dimension {index_vector_dim=} of "
                     f"indices is {indices.shape[index_vector_dim]}. These two "
                     f"numbers must be equal")
 
@@ -1563,7 +1560,7 @@ def _clamp_scatter_indices(operand, indices, updates, *, dnums):
 
   upper_bounds: core.Shape = tuple(operand.shape[i] - slice_sizes[i]
                                    for i in dnums.scatter_dims_to_operand_dims)
-  # Stack upper_bounds into a DeviceArray[n]
+  # Stack upper_bounds into a Array[n]
   upper_bound = lax.shape_as_value(upper_bounds)
   upper_bound = lax.min(upper_bound, np.iinfo(indices.dtype).max)
   upper_bound = lax.broadcast_in_dim(upper_bound, indices.shape,
@@ -2046,7 +2043,8 @@ def _scatter_lower(ctx, operand, indices, updates, *,
       raise NotImplementedError('Cannot lower effectful `scatter`.')
     out_nodes, _ = mlir.jaxpr_subcomp(
         update_ctx, update_jaxpr, mlir.TokenSet(), update_consts,
-        (update.arguments[0],), (update.arguments[1],))
+        (update.arguments[0],), (update.arguments[1],),
+        dim_var_values=ctx.dim_var_values)
     mhlo.ReturnOp(util.flatten(out_nodes))
   return op.results
 
