@@ -154,6 +154,10 @@ def _wraps(
       be determined from the wrapped function itself.
   """
   def wrap(op):
+    op.__np_wrapped__ = fun
+    # Allows this pattern: @wraps(getattr(np, 'new_function', None))
+    if fun is None:
+      return op
     docstr = getattr(fun, "__doc__", None)
     name = getattr(fun, "__name__", getattr(op, "__name__", str(op)))
     try:
@@ -173,12 +177,18 @@ def _wraps(
           parameters = _parse_parameters(parsed.sections['Parameters'])
           if extra_params:
             parameters.update(_parse_extra_params(extra_params))
-          parsed.sections['Parameters'] = (
-            "Parameters\n"
-            "----------\n" +
-            "\n".join(_versionadded.split(desc)[0].rstrip() for p, desc in parameters.items()
-                      if (code is None or p in code.co_varnames) and p not in skip_params)
-          )
+          parameters = {p: desc for p, desc in parameters.items()
+                        if (code is None or p in code.co_varnames)
+                        and p not in skip_params}
+          if parameters:
+            parsed.sections['Parameters'] = (
+              "Parameters\n"
+              "----------\n" +
+              "\n".join(_versionadded.split(desc)[0].rstrip()
+                        for p, desc in parameters.items())
+            )
+          else:
+            del parsed.sections['Parameters']
 
         docstr = parsed.summary.strip() + "\n" if parsed.summary else ""
         docstr += f"\nLAX-backend implementation of :func:`{name}`.\n"
@@ -203,7 +213,6 @@ def _wraps(
         docstr = fun.__doc__
 
     op.__doc__ = docstr
-    op.__np_wrapped__ = fun
     for attr in ['__name__', '__qualname__']:
       try:
         value = getattr(fun, attr)
@@ -411,8 +420,7 @@ def _broadcast_to(arr: ArrayLike, shape: Shape) -> Array:
 # `np.where(np.zeros(1000), 7, 4)`. In op-by-op mode, we don't want to
 # materialize the broadcast forms of scalar arguments.
 @api.jit
-def _where(condition: ArrayLike, x: Optional[ArrayLike] = None,
-           y: Optional[ArrayLike] = None) -> Array:
+def _where(condition: ArrayLike, x: ArrayLike, y: ArrayLike) -> Array:
   if x is None or y is None:
     raise ValueError("Either both or neither of the x and y arguments should "
                      "be provided to jax.numpy.where, got {} and {}."
