@@ -104,7 +104,7 @@ def _check_prng_key_data(impl, key_data: jnp.ndarray):
                     f"key_data.ndim >= 1; got ndim={key_data.ndim}")
   if key_data.shape[-ndim:] != impl.key_shape:
     raise TypeError("JAX encountered invalid PRNG key data: expected key_data.shape to "
-                    f"end with {impl.key_shape}; got shape={key_data.shape} for impl={impl}")
+                    f"end with {impl.key_shape}; got shape={key_data.shape} for {impl=}")
   if key_data.dtype not in [np.uint32, float0]:
     raise TypeError("JAX encountered invalid PRNG key data: expected key_data.dtype = uint32; "
                     f"got dtype={key_data.dtype}")
@@ -282,13 +282,13 @@ def base_arr_shape_to_keys_shape(impl, base_arr_shape):
 class KeyTyRules:
 
   @staticmethod
-  def physical_avals(aval):  # TODO(frostig): rename to `grounded_avals`
+  def physical_avals(aval) -> Sequence[core.AbstractValue]:  # TODO(frostig): rename to `grounded_avals`
     # TODO(frostig): dedup with `keys_aval_to_base_arr_aval``
-    return [core.ShapedArray((*aval.shape, *aval.dtype.impl.key_shape),
+    return [core.ShapedArray((*aval.shape, *aval.dtype.impl.key_shape),  # type: ignore
                              jnp.dtype('uint32'))]
 
   @staticmethod
-  def aval_to_ir_types(aval):
+  def aval_to_ir_types(aval: core.AbstractValue) -> Sequence[mlir.ir.Type]:
     phys_aval, = KeyTyRules.physical_avals(aval)
     return mlir.aval_to_ir_types(phys_aval)
 
@@ -393,70 +393,64 @@ class KeyTyRules:
   # element-type-polymorphic primitive lowering rules
 
   @staticmethod
-  def empty_mlir(ctx):
-    aval_out, = ctx.avals_out
+  def empty_mlir(ctx, aval_out) -> Sequence[mlir.ir.Value]:
     return mlir.ir_constants(np.zeros(aval_out.dtype.impl.key_shape,
                                       dtype=np.dtype('uint32')))
 
   @staticmethod
-  def slice_mlir(ctx, x, start_indices, limit_indices, strides):
-    aval_out, = ctx.avals_out
+  def slice_mlir(ctx, aval_out, x, start_indices, limit_indices, strides) -> mlir.ir.Value:
     key_shape = aval_out.dtype.impl.key_shape
     trailing_zeros = [0] * len(key_shape)
     trailing_ones  = [1] * len(key_shape)
     start_indices = (*start_indices, *trailing_zeros)
     limit_indices = (*limit_indices, *key_shape)
     strides = (*strides, *trailing_ones)
-    return mhlo.SliceOp(x,
-                        mlir.dense_int_elements(start_indices),
-                        mlir.dense_int_elements(limit_indices),
-                        mlir.dense_int_elements(strides)).results
+    physical_aval_out, = KeyTyRules.physical_avals(aval_out)
+    return mlir.slice_op(ctx, x, physical_aval_out,
+                         start_indices=start_indices, limit_indices=limit_indices, strides=strides)
 
   @staticmethod
-  def dynamic_slice_mlir(ctx, x, start_indices, slice_sizes):
-    aval_out, = ctx.avals_out
+  def dynamic_slice_mlir(ctx, aval_out, x, start_indices) -> mlir.ir.Value:
     dtype = dtypes.canonicalize_dtype(np.dtype('int64'))
     key_shape = aval_out.dtype.impl.key_shape
     trailing_zeros = [mlir.ir_constant(np.array(0, dtype))] * len(key_shape)
     start_indices = (*start_indices, *trailing_zeros)
-    slice_sizes_ = mlir.dense_int_elements((*slice_sizes, *key_shape))
-    return mhlo.DynamicSliceOp(x, start_indices, slice_sizes_).results
+    physical_aval_out, = KeyTyRules.physical_avals(aval_out)
+    return mlir.dynamic_slice(ctx, physical_aval_out, x,
+                              start_indices=start_indices)
 
   @staticmethod
-  def dynamic_update_slice_mlir(ctx, x, update, *start_indices):
-    aval_out, = ctx.avals_out
+  def dynamic_update_slice_mlir(ctx, aval_out, x, update, *start_indices) -> mlir.ir.Value:
     dtype = dtypes.canonicalize_dtype(np.dtype('int64'))
     key_shape = aval_out.dtype.impl.key_shape
     zeros = [mlir.ir_constant(np.array(0, dtype=dtype))] * len(key_shape)
     start_indices = (*start_indices, *zeros)
-    return mhlo.DynamicUpdateSliceOp(mlir.aval_to_ir_type(aval_out), x, update,
-                                     start_indices).results
+    physical_aval_out, = KeyTyRules.physical_avals(aval_out)
+    return mlir.dynamic_update_slice(ctx, physical_aval_out, x, update,
+                                     start_indices=start_indices)
 
   @staticmethod
-  def broadcast_in_dim_mlir(ctx, x, *dyn_shape, shape, broadcast_dimensions):
-    if dyn_shape: raise NotImplementedError
-    aval_out, = ctx.avals_out
+  def broadcast_in_dim_mlir(ctx, aval_out, x,
+                            broadcast_dimensions) -> mlir.ir.Value:
     key_shape = aval_out.dtype.impl.key_shape
     trailing_dims = [aval_out.ndim + i for i in range(len(key_shape))]
     broadcast_dimensions = [*broadcast_dimensions, *trailing_dims]
-    return mhlo.BroadcastInDimOp(
-        mlir.aval_to_ir_type(aval_out), x,
-        mlir.dense_int_elements(broadcast_dimensions)).results
+    physical_aval_out, = KeyTyRules.physical_avals(aval_out)
+    return mlir.broadcast_in_dim(ctx, x, physical_aval_out, broadcast_dimensions=broadcast_dimensions)
 
   @staticmethod
-  def transpose_mlir(ctx, x, *, permutation):
-    aval_out, = ctx.avals_out
+  def transpose_mlir(ctx, aval_out, x, *, permutation) -> mlir.ir.Value:
     key_shape = aval_out.dtype.impl.key_shape
     trailing_dims = [aval_out.ndim + i for i in range(len(key_shape))]
     perm = [*permutation, *trailing_dims]
-    return mhlo.TransposeOp(x, mlir.dense_int_elements(perm)).results
+    return mhlo.TransposeOp(x, mlir.dense_int_elements(perm)).result
 
   @staticmethod
-  def gather_mlir(ctx, x, indices, *,
+  def gather_mlir(ctx, avals_in, aval_out, x, indices, *,
                   dimension_numbers, slice_sizes, unique_indices,
-                  indices_are_sorted, mode, fill_value):
-    aval_x, aval_indices = ctx.avals_in
-    aval_y, = ctx.avals_out
+                  indices_are_sorted, mode, fill_value) -> mlir.ir.Value:
+    aval_x, aval_indices = avals_in
+    aval_y = aval_out
     key_shape = aval_x.dtype.impl.key_shape
     trailing_offset_dims = [aval_y.ndim + i for i in range(len(key_shape))]
     dimension_numbers = dimension_numbers._replace(
@@ -466,10 +460,11 @@ class KeyTyRules:
         lax_internal.slicing._gather_lower, dimension_numbers=dimension_numbers,
         slice_sizes=slice_sizes, unique_indices=unique_indices,
         indices_are_sorted=indices_are_sorted, mode=mode, fill_value=fill_value)
-    return mlir.delegate_lowering(
+    res, = mlir.delegate_lowering(
         ctx, gather_lower, x, indices,
         avals_in=[keys_aval_to_base_arr_aval(aval_x), aval_indices],
         avals_out=[keys_aval_to_base_arr_aval(aval_y)])
+    return res
 
 
 class KeyTy:
@@ -950,12 +945,11 @@ def _threefry2x32_gpu_lowering(threefry2x32_lowering, ctx, k1, k2, x1, x2):
   k1_aval, k2_aval, x1_aval, x2_aval = ctx.avals_in
   rank = len(aval_out.shape)
   if 0 in aval_out.shape:
-    zeros = mlir.full_like_aval(0, aval_out)
+    zeros = mlir.full_like_aval(ctx, 0, aval_out)
     return [zeros, zeros]
   def _broadcast(x, aval):
-    return mhlo.BroadcastInDimOp(
-        mlir.aval_to_ir_type(aval_out), x,
-        mlir.dense_int_elements(range(rank - len(aval.shape), rank))).result
+    return mlir.broadcast_in_dim(ctx, x, aval_out,
+                                 broadcast_dimensions=range(rank - len(aval.shape), rank))
   return threefry2x32_lowering(
           (_broadcast(k1, k1_aval), _broadcast(k2, k2_aval)),
           (_broadcast(x1, x1_aval), _broadcast(x2, x2_aval)))
@@ -982,20 +976,70 @@ mlir.register_lowering(
     platform='rocm')
 
 
-def iota_32x2_shape(shape):
+def iota_2x32_shape(shape):
+  """Reshaped ``uint64`` iota, as two parallel ``uint32`` arrays.
+
+  Setting aside representation, this function essentially computes the
+  equivalent of::
+
+    jax.lax.iota(dtype=np.uint64, size=np.prod(shape)).reshape(shape)
+
+  However:
+
+  * It returns two parallel ``uint32`` arrays instead of one
+    ``uint64`` array. This renders it invariant under either setting of
+    the system-wide ``jax_enable_x64`` configuration flag.
+
+  * It lowers in a way such that the compiler's automatic SPMD
+    partitioner recognizes its partitionability.
+
+  For example::
+
+    >>> import numpy as np
+    >>> from jax import lax
+    >>> from jax._src import prng
+
+    >>> prng.iota_2x32_shape((3, 4))
+    [Array([[0, 0, 0, 0],
+            [0, 0, 0, 0],
+            [0, 0, 0, 0]], dtype=uint32),
+     Array([[ 0,  1,  2,  3],
+            [ 4,  5,  6,  7],
+            [ 8,  9, 10, 11]], dtype=uint32)]
+
+    >>> def reshaped_iota(shape):
+    ...   return lax.iota(size=np.prod(shape), dtype=np.uint32).reshape(shape)
+    ...
+    >>> reshaped_iota((3, 4))
+    Array([[ 0,  1,  2,  3],
+           [ 4,  5,  6,  7],
+           [ 8,  9, 10, 11]], dtype=uint32)
+
+  Args:
+    shape: the output shape
+
+  Returns:
+    A pair of ``uint32`` arrays ``(counts_hi, counts_lo)``, both of
+    shape ``shape``, representing the higher-order and lower-order 32
+    bits of the 64 bit unsigned iota.
+  """
   if len(shape) == 0:
     return (jnp.zeros((), np.dtype('uint32')),) * 2
-  return iota_32x2_shape_p.bind(shape=shape)
+  return iota_2x32_shape_p.bind(shape=shape)
 
-iota_32x2_shape_p = core.Primitive('iota_32x2_shape')
-iota_32x2_shape_p.multiple_results = True
-iota_32x2_shape_p.def_impl(partial(xla.apply_primitive, iota_32x2_shape_p))
+iota_2x32_shape_p = core.Primitive('iota_2x32_shape')
+iota_2x32_shape_p.multiple_results = True
+iota_2x32_shape_p.def_impl(partial(xla.apply_primitive, iota_2x32_shape_p))
 
-@iota_32x2_shape_p.def_abstract_eval
-def iota_32x2_shape_abstract_eval(*, shape):
+@iota_2x32_shape_p.def_abstract_eval
+def iota_2x32_shape_abstract_eval(*, shape):
   return (core.ShapedArray(shape, np.dtype('uint32')),) * 2
 
-def iota_32x2_shape_lowering(ctx, *, shape):
+def bcast_iotas_to_reshaped_iota(add, mul, shape, iotas):
+  strides = (*map(int, np.cumprod(shape[1:][::-1])[::-1]), 1)
+  return reduce(add, [mul(s, i) for i, s in zip(iotas, strides)])  # type: ignore
+
+def iota_2x32_shape_lowering(ctx, *, shape):
   def _add(x, y):
     return mlir.mhlo.AddOp(x, y).result
 
@@ -1005,17 +1049,13 @@ def iota_32x2_shape_lowering(ctx, *, shape):
     x_bcast = mlir.mhlo.BroadcastOp(x_const, mlir.dense_int_elements(shape))
     return mlir.mhlo.MulOp(x_bcast, y).result
 
-  def _sum(xs):
-    return reduce(_add, xs)
-
   assert len(shape) > 0
   aval_out, _ = ctx.avals_out
   aval_u64 = core.ShapedArray(shape, np.dtype('uint64'))
   iotas = [mlir.mhlo.IotaOp(mlir.aval_to_ir_type(aval_u64),
-                       mlir.i64_attr(dimension)).result
+                            mlir.i64_attr(dimension)).result
            for dimension in range(len(shape))]
-  strides = (*map(int, np.cumprod(shape[1:][::-1])[::-1]), 1)
-  counts = _sum(_mul(s, i) for i, s in zip(iotas, strides))  # type: ignore
+  counts = bcast_iotas_to_reshaped_iota(_add, _mul, shape, iotas)
   shift = mlir.ir_constant(np.array(32, np.dtype('uint64')),
                            canonicalize_types=False)
   shift = mlir.mhlo.BroadcastOp(shift, mlir.dense_int_elements(shape)).result
@@ -1023,8 +1063,8 @@ def iota_32x2_shape_lowering(ctx, *, shape):
   counts_lo = mlir.mhlo.ConvertOp(mlir.aval_to_ir_type(aval_out), counts).result
   counts_hi = mlir.mhlo.ConvertOp(mlir.aval_to_ir_type(aval_out),
                                   counts_shifted).result
-  return (counts_hi, counts_lo)
-mlir.register_lowering(iota_32x2_shape_p, iota_32x2_shape_lowering)
+  return counts_hi, counts_lo
+mlir.register_lowering(iota_2x32_shape_p, iota_2x32_shape_lowering)
 
 
 @partial(jit, inline=True)
@@ -1062,13 +1102,23 @@ def threefry_2x32(keypair, count):
 
 
 def threefry_split(key: jnp.ndarray, num: int) -> jnp.ndarray:
-  return _threefry_split(key, int(num))  # type: ignore
+  if config.jax_threefry_partitionable:
+    return _threefry_split_foldlike(key, int(num))  # type: ignore
+  else:
+    return _threefry_split_original(key, int(num))  # type: ignore
 
 @partial(jit, static_argnums=(1,), inline=True)
-def _threefry_split(key, num) -> jnp.ndarray:
+def _threefry_split_original(key, num) -> jnp.ndarray:
   counts = lax.iota(np.uint32, num * 2)
   return lax.reshape(threefry_2x32(key, counts), (num, 2))
 
+@partial(jit, static_argnums=(1,), inline=True)
+def _threefry_split_foldlike(key, num) -> jnp.ndarray:
+  k1, k2 = key
+  counts1, counts2 = iota_2x32_shape((num,))
+  bits1, bits2 = threefry2x32_p.bind(k1, k2, counts1, counts2)
+  return jnp.stack([bits1, bits2], axis=1)
+
 
 def threefry_fold_in(key: jnp.ndarray, data: jnp.ndarray) -> jnp.ndarray:
   assert not data.shape
@@ -1097,7 +1147,7 @@ def _threefry_random_bits_partitionable(key: jnp.ndarray, bit_width, shape):
     raise NotImplementedError('random bits array of size exceeding 2 ** 64')
 
   k1, k2 = key
-  counts1, counts2 = iota_32x2_shape(shape)
+  counts1, counts2 = iota_2x32_shape(shape)
   bits1, bits2 = threefry2x32_p.bind(k1, k2, counts1, counts2)
 
   dtype = UINT_DTYPES[bit_width]
@@ -1177,7 +1227,12 @@ def _rbg_seed(seed: jnp.ndarray) -> jnp.ndarray:
   return jnp.concatenate([halfkey, halfkey])
 
 def _rbg_split(key: jnp.ndarray, num: int) -> jnp.ndarray:
-  return vmap(_threefry_split, (0, None), 1)(key.reshape(2, 2), num).reshape(num, 4)
+  if config.jax_threefry_partitionable:
+    _threefry_split = _threefry_split_foldlike
+  else:
+    _threefry_split = _threefry_split_original
+  return vmap(
+      _threefry_split, (0, None), 1)(key.reshape(2, 2), num).reshape(num, 4)
 
 def _rbg_fold_in(key: jnp.ndarray, data: jnp.ndarray) -> jnp.ndarray:
   assert not data.shape
