@@ -21,9 +21,10 @@ from typing import (Sequence, List, Tuple, Optional, Mapping, Dict, Set,
                     FrozenSet, Union, cast)
 
 import jax
-from jax._src.util import safe_map, safe_zip
+from jax._src.util import safe_map, safe_zip, use_cpp_class, use_cpp_method
 from jax._src.lib import xla_client as xc
-from jax.interpreters import pxla, mlir
+from jax.interpreters import mlir
+from jax.interpreters import pxla
 
 import numpy as np
 
@@ -32,8 +33,7 @@ Device = xc.Device
 Index = Tuple[slice, ...]
 XLADeviceAssignment = Sequence[Device]
 
-
-@pxla.use_cpp_class(xc.Sharding if xc._version >= 94 else None)
+@use_cpp_class(xc.Sharding if xc._version >= 94 else None)
 class Sharding(metaclass=abc.ABCMeta):
   """Abstract ``Sharding`` interface which describes how a ``jax.Array`` is laid out
   across devices.
@@ -72,13 +72,13 @@ class Sharding(metaclass=abc.ABCMeta):
   #############################################################################
   # Default implementations below that all subclasses will inherit.
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def addressable_devices(self) -> Set[Device]:
     """A set of devices that are addressable by the current process."""
     return {d for d in self.device_set
             if d.process_index == d.client.process_index()}
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def is_fully_addressable(self) -> bool:
     """True if the current process can address all of the devices in device_set.
     """
@@ -99,7 +99,7 @@ class Sharding(metaclass=abc.ABCMeta):
 
 # Shardings that inherit from XLACompatibleSharding should implement the
 # `_device_assignment` property and `_to_xla_op_sharding` method.
-@pxla.use_cpp_class(xc.XLACompatibleSharding if xc._version >= 94 else None)
+@use_cpp_class(xc.XLACompatibleSharding if xc._version >= 94 else None)
 class XLACompatibleSharding(Sharding, metaclass=abc.ABCMeta):
   """A `Sharding` that describes shardings expressible to XLA.
 
@@ -127,7 +127,7 @@ class XLACompatibleSharding(Sharding, metaclass=abc.ABCMeta):
                                               op_sharding)
     return op_sharding_sharding.devices_indices_map(global_shape)
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def _addressable_device_assignment(self) -> XLADeviceAssignment:
     return [d for d in self._device_assignment
             if d.process_index == d.client.process_index()]
@@ -199,9 +199,45 @@ def _enable_cpp_named_sharding():
     return None
 
 
-@pxla.use_cpp_class(_enable_cpp_named_sharding())
+class _UnconstrainedPartitionSingleton:
+
+  def __str__(self):
+    return "UNCONSTRAINED"
+
+
+# Unconstrained sentinel value for PartitionSpec, representing a dimension for
+# which the user wants XLA to assign the best partitioning.
+# TODO(yashkatariya): May rename to AUTO.
+_UNCONSTRAINED_PARTITION = _UnconstrainedPartitionSingleton()
+
+
+class PartitionSpec(tuple):
+  """Tuple of integer specifying how a value should be partitioned.
+
+  Each integer corresponds to how many ways a dimension is partitioned. We
+  create a separate class for this so JAX's pytree utilities can distinguish it
+  from a tuple that should be treated as a pytree.
+  """
+
+  # A sentinel value representing a dim is unconstrained.
+  UNCONSTRAINED = _UNCONSTRAINED_PARTITION
+
+  def __init__(self, *partitions):
+    pass
+
+  def __new__(cls, *partitions):
+    return tuple.__new__(PartitionSpec, partitions)
+
+  def __repr__(self):
+    return "PartitionSpec%s" % tuple.__repr__(self)
+
+  def __reduce__(self):
+    return (PartitionSpec, tuple(self))
+
+
+@use_cpp_class(_enable_cpp_named_sharding())
 class NamedSharding(XLACompatibleSharding):
-  """NamedSharding is a way to express ``Sharding``s using named axes.
+  r"""NamedSharding is a way to express ``Sharding``\s using named axes.
 
   ``Mesh`` and ``PartitionSpec`` can be used to express a ``Sharding`` with a name.
 
@@ -233,21 +269,17 @@ class NamedSharding(XLACompatibleSharding):
     >>> named_sharding = jax.sharding.NamedSharding(mesh, spec)
   """
 
-  @pxla.use_cpp_method
+  @use_cpp_method
   def __init__(
-      self, mesh: pxla.Mesh, spec: pxla.PartitionSpec, _parsed_pspec = None):
+      self, mesh: pxla.Mesh, spec: PartitionSpec, _parsed_pspec = None):
 
     self.mesh = mesh
     self.spec = spec
     self._parsed_pspec = _parsed_pspec
     self._preprocess()
 
-  @classmethod
-  def unpickle(cls, mesh, spec):
-    return cls(mesh, spec)
-
   def __reduce__(self):
-    return type(self).unpickle, (self.mesh, self.spec)
+    return type(self), (self.mesh, self.spec)
 
   def _preprocess(self):
     # This split exists because you can pass `_parsed_pspec` that has been
@@ -265,7 +297,7 @@ class NamedSharding(XLACompatibleSharding):
     _check_mesh_resource_axis(self.mesh, self._parsed_pspec)
 
   def __repr__(self):
-    return f'NamedSharding(mesh={dict(self.mesh.shape)}, partition_spec={self.spec})'
+    return f'NamedSharding(mesh={dict(self.mesh.shape)}, spec={self.spec})'
 
   def __hash__(self):
     if not hasattr(self, '_hash'):
@@ -292,11 +324,11 @@ class NamedSharding(XLACompatibleSharding):
   def _from_parsed_pspec(cls, mesh, parsed_pspec):
     return cls(mesh, parsed_pspec.get_partition_spec(), parsed_pspec)
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def device_set(self) -> Set[Device]:
     return set(self.mesh.devices.flat)
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def _device_assignment(self) -> XLADeviceAssignment:
     return list(self.mesh.devices.flat)
 
@@ -335,7 +367,7 @@ def _get_replicated_op_sharding():
   return proto
 
 
-@pxla.use_cpp_class(xc.SingleDeviceSharding if xc._version >= 95 else None)
+@use_cpp_class(xc.SingleDeviceSharding if xc._version >= 95 else None)
 class SingleDeviceSharding(XLACompatibleSharding):
   """A subclass of ``XLACompatibleSharding`` that places its data on a single device.
 
@@ -348,16 +380,12 @@ class SingleDeviceSharding(XLACompatibleSharding):
     ...     jax.devices()[0])
   """
 
-  @pxla.use_cpp_method
+  @use_cpp_method
   def __init__(self, device: Device):
     self._device = device
 
-  @classmethod
-  def unpickle(cls, device: Device):
-    return cls(device)
-
   def __reduce__(self):
-    return type(self).unpickle, (self._device,)
+    return type(self), (self._device,)
 
   def __repr__(self):
     return f"SingleDeviceSharding(device={repr(self._device)})"
@@ -387,21 +415,17 @@ class SingleDeviceSharding(XLACompatibleSharding):
     return _get_replicated_op_sharding()
 
 
-@pxla.use_cpp_class(xc.PmapSharding if xc._version >= 94 else None)
+@use_cpp_class(xc.PmapSharding if xc._version >= 94 else None)
 class PmapSharding(XLACompatibleSharding):
 
-  @pxla.use_cpp_method
+  @use_cpp_method
   def __init__(self, devices: np.ndarray, sharding_spec: pxla.ShardingSpec):
     self.devices = devices
     # The sharding spec should be pmap's sharding spec.
     self.sharding_spec = sharding_spec
 
-  @classmethod
-  def unpickle(cls, devices: np.ndarray, sharding_spec: pxla.ShardingSpec):
-    return cls(devices, sharding_spec)
-
   def __reduce__(self):
-    return type(self).unpickle, (self.devices, self.sharding_spec)
+    return type(self), (self.devices, self.sharding_spec)
 
   def __eq__(self, other):
     if not isinstance(other, PmapSharding):
@@ -419,7 +443,7 @@ class PmapSharding(XLACompatibleSharding):
   def __str__(self):
     device_ids = [d.id for d in self.devices.flat]
     return (f'PmapSharding(sharding_spec={self.sharding_spec}, '
-            f'device_ids={device_ids}, '
+            f'{device_ids=}, '
             f'device_platform={self.devices.flat[0].platform.upper()}, '
             f'device_shape={self.devices.shape})')
 
@@ -454,7 +478,7 @@ class PmapSharding(XLACompatibleSharding):
     pmap_devices = jax.local_devices()[:num_ways_sharded]
     return cls(pmap_devices, sharding_spec)
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def device_set(self) -> Set[Device]:
     return set(self.devices.flat)
 
@@ -463,7 +487,7 @@ class PmapSharding(XLACompatibleSharding):
     indices = pxla.spec_to_indices(global_shape, self.sharding_spec)
     return dict(safe_zip(self.devices.flat, indices))  # type: ignore[arg-type]
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def _device_assignment(self) -> XLADeviceAssignment:
     return list(self.devices.flat)
 
@@ -541,7 +565,7 @@ class PositionalSharding(XLACompatibleSharding):
 
   # Sharding interface
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def device_set(self) -> set[xc.Device]:
     return set(self._devices)
 
@@ -600,22 +624,18 @@ class DeviceIdSet:
             self._ids == other._ids)
 
 
-@pxla.use_cpp_class(xc.OpShardingSharding if xc._version >= 95 else None)
+@use_cpp_class(xc.OpShardingSharding if xc._version >= 95 else None)
 class OpShardingSharding(XLACompatibleSharding):
 
-  @pxla.use_cpp_method
+  @use_cpp_method
   def __init__(self, devices: Sequence[Device], op_sharding: xc.OpSharding):
     self._devices = tuple(devices)
     self._op_sharding = op_sharding
 
-  @classmethod
-  def unpickle(cls, devices: Sequence[Device], op_sharding: xc.OpSharding):
-    return cls(devices, op_sharding)
-
   def __reduce__(self):
-    return type(self).unpickle, (self._devices, self._op_sharding)
+    return type(self), (self._devices, self._op_sharding)
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def _op_sharding_hash(self):
     return hash(xc.HloSharding.from_proto(self._op_sharding))
 
@@ -643,7 +663,7 @@ class OpShardingSharding(XLACompatibleSharding):
           f"{len(num_ways_dim_sharded)}, but was applied to a value of rank "
           f"{len(aval_shape)}")
 
-  @pxla.maybe_cached_property
+  @functools.cached_property
   def device_set(self) -> Set[Device]:
     return set(self._devices)
 
