@@ -769,11 +769,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     arg = np.array(3.)
     f_tf = jax2tf.convert(jax.grad(remat_f))
     f_tf_hlo = self.TfToHlo(f_tf, arg)
-    if jax.config.jax_remat_opt_barrier:
-      self.assertRegex(f_tf_hlo, r"opt-barrier")
-    else:
-      self.assertRegex(f_tf_hlo,
-                       r'transpose/jax2tf_f_/jvp/checkpoint/cond/branch_1_fun/Sin')
+    self.assertRegex(f_tf_hlo, r"opt-barrier")
 
   def test_remat_free_var(self):
     def f(x):
@@ -1212,105 +1208,6 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
         include_xla_op_metadata=False
     )
 
-def get_serialized_computation(
-    f_jax: Callable,
-    *args,
-    abstracted_axes: Optional[Tuple[Dict[int, str]]] = None,
-    use_pjit: bool = False,
-    in_axis_resources = None,
-    out_axis_resources = None) -> str:
-  if use_pjit:
-    assert not abstracted_axes
-    lowered = pjit(f_jax,
-                   in_axis_resources=in_axis_resources,
-                   out_axis_resources=out_axis_resources).lower(*args)
-  else:
-    lowered = jax.jit(f_jax, abstracted_axes=abstracted_axes).lower(*args)
-  mhlo_module = lowered.compiler_ir(dialect='mhlo')
-  mhlo_module_text = mlir.module_to_string(mhlo_module)
-  logging.info(f'Serialized ir.Module = {mhlo_module_text}')
-  return mhlo_module_text
-
-
-class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
-  """Unit tests for XlaCallModule. Will move these eventually to TF."""
-  def test_simple(self):
-
-    def f_jax(x):
-      return jnp.sin(x)
-
-    x = np.ones((2, 3), dtype=np.float32)
-
-    jax_res = f_jax(x)
-    res = tfxla.call_module([x],
-                            module=get_serialized_computation(f_jax, x),
-                            Tout=[jax_res.dtype],
-                            Sout=[jax_res.shape])
-    self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
-                        [jax_res])
-
-  def test_while(self):
-    # With nested computation
-    def f_jax(count, x):
-      return lax.while_loop(lambda carry: carry[0] < count, lambda carry:
-                            (carry[0] + 1, carry[1] + 1.), (0, x))[1]
-
-    count = np.int32(5)
-    x = np.ones((2, 3), dtype=np.float32)
-
-    jax_res = f_jax(count, x)
-    res = tfxla.call_module([count, x],
-                            module=get_serialized_computation(f_jax, count, x),
-                            Tout=[jax_res.dtype],
-                            Sout=[jax_res.shape])
-    self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
-                        [jax_res])
-
-  def test_multiple_args_results(self):
-
-    def f_jax(x1, x2):
-      return (jnp.sin(x1), jnp.cos(x2))
-
-    x1 = np.ones((2, 3), dtype=np.float32)
-    x2 = np.ones((3, 4), dtype=np.float32)
-
-    jax_res = f_jax(x1, x2)
-
-    def f_tf(x1_tf, x2_tf):
-      return tfxla.call_module([x1_tf, x2_tf],
-                               module=get_serialized_computation(f_jax, x1, x2),
-                               Tout=[jax_res[0].dtype, jax_res[1].dtype],
-                               Sout=[jax_res[0].shape, jax_res[1].shape])
-
-    res = tf.function(f_tf, jit_compile=True, autograph=False)(x1, x2)
-    self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
-                        jax_res)
-
-  @unittest.skip("TODO(necula): 'mhlo.dynamic_iota' op can't be translated to XLA HLO")
-  def test_shape_poly_arange(self):
-    if not config.jax_dynamic_shapes:
-      raise unittest.SkipTest("jax_dynamic_shapes must be enabled")
-    def f_jax(x):  # x: f32[b]
-      return jnp.arange(x.shape[0]) + x
-
-    x1 = np.ones((5,), dtype=np.float32)
-    jax_res = f_jax(x1)
-
-    def f_tf(x1_tf):
-      return tfxla.call_module([x1_tf],
-                               module=get_serialized_computation(
-                                   f_jax, x1,
-                                   abstracted_axes=({
-                                       0: 'b'
-                                   },)),
-                               Tout=[jax_res.dtype],
-                               Sout=[jax_res.shape],
-                               dim_args_spec=('0.0',))
-
-    res = tf.function(f_tf, jit_compile=True, autograph=False)(x1)
-    self.assertAllClose(
-        tf.nest.map_structure(lambda t: t.numpy(), res), jax_res)
-
   # TODO(necula): figure out this failure
   @jtu.skip_on_flag("jax2tf_default_experimental_native_lowering", True)
   def test_global_device_array(self):
@@ -1347,6 +1244,7 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
 
   @jtu.with_mesh([("x", 2)])
   def test_pjit_basic1D(self):
+
     def func_jax(x, y):
       return x + y
 
@@ -1354,51 +1252,237 @@ class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
     x = np.arange(np.prod(shape), dtype=np.float32).reshape(shape)
     in_axis_resources = (P("x"), P("x"))
     out_axis_resources = None
-    res_jax = pjit(func_jax,
-                   in_axis_resources=in_axis_resources,
-                   out_axis_resources=out_axis_resources)(x, x)
-    module = get_serialized_computation(func_jax, x, x,
-                                        use_pjit=True,
-                                        in_axis_resources=in_axis_resources,
-                                        out_axis_resources=out_axis_resources)
+    res_jax = pjit(
+        func_jax,
+        in_axis_resources=in_axis_resources,
+        out_axis_resources=out_axis_resources)(x, x)
+    module = get_serialized_computation(
+        func_jax,
+        x,
+        x,
+        use_pjit=True,
+        in_axis_resources=in_axis_resources,
+        out_axis_resources=out_axis_resources)
+
     def f_tf(x_tf, y_tf):
       return tfxla.call_module([x_tf, y_tf],
+                               version=2,
                                module=module,
                                Tout=[x.dtype],
                                Sout=[x.shape])
+
     res_tf = tf.function(f_tf, jit_compile=True, autograph=False)(x, x)[0]
     self.assertAllClose(res_tf.numpy(), res_jax)
 
-  def test_outer_name_scope(self):
+  def assertAllOperationStartWith(self, g: tf.Graph, scope_name: str):
+    """Assert all operations name start with ```scope_name```.
+
+    Also the scope_name only occur one time.
+    """
+    result = g.get_operations()
+    if not result:
+      self.fail("result is empty.")
+    for op in result:
+      logging.info("tf op.name = %s", op.name)
+      if not op.name.startswith(scope_name):
+        self.fail(f"{op.name} does not start with {scope_name}.")
+
+  def test_name_scope_polymorphic(self):
     if config.jax2tf_default_experimental_native_lowering and not config.jax_dynamic_shapes:
       self.skipTest("shape polymorphism but --jax_dynamic_shapes is not set.")
 
-    def assertAllOperationStartWith(g: tf.Graph, scope_name):
-      """Assert all operations name start with ```scope_name```."""
-      result = g.get_operations()
-      if not result:
-        self.fail("result is empty.")
-      for op in result:
-        logging.info("op.name = %s", op.name)
-        if not op.name.startswith(scope_name):
-          self.fail(f"{op.name} does not start with {scope_name}.")
-
     def func_jax(x, y):
       return jnp.sin(x) + jnp.cos(y)
 
+    func_tf = jax2tf.convert(
+        func_jax, polymorphic_shapes="(b,...)", with_gradient=True)
+
     outer_scope = "output_a"
+
     g = tf.Graph()
     with g.as_default() as g:
       with tf.name_scope(outer_scope):
         x = tf.Variable(
             tf.zeros(shape=(1, 5), dtype=tf.dtypes.float32), name="x")
         y = tf.compat.v1.placeholder(tf.dtypes.float32, (None, 5), "y")
-        func_tf = jax2tf.convert(func_jax, polymorphic_shapes="(b,...)")
         _ = func_tf(x, y)
-    assertAllOperationStartWith(g, outer_scope)
+    self.assertAllOperationStartWith(g, outer_scope)
+
+    # wrap tf.function
+    g2 = tf.Graph()
+    with g2.as_default() as g:
+      with tf.name_scope(outer_scope):
+        x = tf.Variable(
+            tf.zeros(shape=(1, 5), dtype=tf.dtypes.float32), name="x")
+        y = tf.compat.v1.placeholder(tf.dtypes.float32, (None, 5), "y")
+        _ = tf.function(func_tf, jit_compile=True, autograph=False)(x, y)
+    self.assertAllOperationStartWith(g2, outer_scope)
+
+  def test_name_scope_cond(self):
+
+    def f(x):
+
+      def f_pos(x):
+        with jax.named_scope("jax_f_pos"):
+          return lax.cond(x < 1., jnp.cos, jnp.sin, x)
+
+      with jax.named_scope("jax_f_outer"):
+        return lax.cond(x > 0., f_pos, lambda x: x, x)
+
+    @tf.function(jit_compile=True, autograph=False)
+    def outer_forward():
+      with tf.name_scope("tf_outer_forward"):
+        x = 0.5
+        f_tf = jax2tf.convert(f)
+        _ = f_tf(x)
+
+    g = outer_forward.get_concrete_function().graph
+    self.assertAllOperationStartWith(g, "tf_outer_forward")
+    for func in g._functions.values():
+      self.assertAllOperationStartWith(
+          func.graph, "tf_outer_forward/jax2tf_f_/jax_f_outer")
+
+    x = tf.Variable(0.5, name="tf_outer_back/x")
+
+    @tf.function(jit_compile=True, autograph=False)
+    def outer_back():
+      with tf.name_scope("tf_outer_back"):
+        f_tf = jax2tf.convert(f)
+        with tf.GradientTape() as tape:
+          res_tf = f_tf(x)
+          _ = tape.gradient(res_tf, x)
+
+    g = outer_back.get_concrete_function().graph
+    self.assertAllOperationStartWith(g, "tf_outer_back")
+    for func in g._functions.values():
+      self.assertAllOperationStartWith(func.graph, "tf_outer_back")
+
+  def test_name_scope_while_loop(self):
+
+    def f(x):
+      with tf.name_scope("outer_scope"):
+
+        def condition(x):
+          return jnp.sum(x, keepdims=False) < 100
+
+        def body(x):
+          return jnp.add(x, 2.0)
+
+        result = jax.lax.while_loop(condition, body, x)
+        return result
+
+    tf_f = tf.function(jax2tf.convert(f), jit_compile=True, autograph=False)
+    g = tf_f.get_concrete_function(tf.zeros((1, 3))).graph
+
+    for func in g._functions.values():
+      for op in func.graph.get_operations():
+        if op.name.count(f"outer_scope/jax2tf_{f.__name__}_/while") > 1:
+          self.fail(
+              "tf graph has repeated name issue on when converting lax.while to tf.while."
+              f"See op.name = : {op.name}")
+
+def get_serialized_computation(
+    f_jax: Callable,
+    *args,
+    abstracted_axes: Optional[Tuple[Dict[int, str]]] = None,
+    use_pjit: bool = False,
+    in_axis_resources = None,
+    out_axis_resources = None) -> str:
+  if use_pjit:
+    assert not abstracted_axes
+    lowered = pjit(f_jax,
+                   in_axis_resources=in_axis_resources,
+                   out_axis_resources=out_axis_resources).lower(*args)
+  else:
+    lowered = jax.jit(f_jax, abstracted_axes=abstracted_axes).lower(*args)
+  stablehlo_module_text = mlir.module_to_string(lowered._lowering.stablehlo())
+  logging.info(f'Serialized ir.Module = {stablehlo_module_text}')
+  return stablehlo_module_text
+
+
+class XlaCallModuleTest(tf_test_util.JaxToTfTestCase):
+  """Unit tests for XlaCallModule. Will move these eventually to TF."""
+  def test_simple(self):
+
+    def f_jax(x):
+      return jnp.sin(x)
+
+    x = np.ones((2, 3), dtype=np.float32)
+
+    jax_res = f_jax(x)
+    res = tfxla.call_module([x],
+                            version=2,
+                            module=get_serialized_computation(f_jax, x),
+                            Tout=[jax_res.dtype],
+                            Sout=[jax_res.shape])
+    self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
+                        [jax_res])
+
+  def test_while(self):
+    # With nested computation
+    def f_jax(count, x):
+      return lax.while_loop(lambda carry: carry[0] < count, lambda carry:
+                            (carry[0] + 1, carry[1] + 1.), (0, x))[1]
+
+    count = np.int32(5)
+    x = np.ones((2, 3), dtype=np.float32)
+
+    jax_res = f_jax(count, x)
+    res = tfxla.call_module([count, x],
+                            version=2,
+                            module=get_serialized_computation(f_jax, count, x),
+                            Tout=[jax_res.dtype],
+                            Sout=[jax_res.shape])
+    self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
+                        [jax_res])
+
+  def test_multiple_args_results(self):
+
+    def f_jax(x1, x2):
+      return (jnp.sin(x1), jnp.cos(x2))
+
+    x1 = np.ones((2, 3), dtype=np.float32)
+    x2 = np.ones((3, 4), dtype=np.float32)
+
+    jax_res = f_jax(x1, x2)
+
+    def f_tf(x1_tf, x2_tf):
+      return tfxla.call_module([x1_tf, x2_tf],
+                               version=2,
+                               module=get_serialized_computation(f_jax, x1, x2),
+                               Tout=[jax_res[0].dtype, jax_res[1].dtype],
+                               Sout=[jax_res[0].shape, jax_res[1].shape])
+
+    res = tf.function(f_tf, jit_compile=True, autograph=False)(x1, x2)
+    self.assertAllClose(tf.nest.map_structure(lambda t: t.numpy(), res),
+                        jax_res)
+
+  @unittest.skip("TODO(necula): 'mhlo.dynamic_iota' op can't be translated to XLA HLO")
+  def test_shape_poly_arange(self):
+    if not config.jax_dynamic_shapes:
+      raise unittest.SkipTest("jax_dynamic_shapes must be enabled")
+    def f_jax(x):  # x: f32[b]
+      return jnp.arange(x.shape[0]) + x
+
+    x1 = np.ones((5,), dtype=np.float32)
+    jax_res = f_jax(x1)
+
+    def f_tf(x1_tf):
+      return tfxla.call_module([x1_tf],
+                               version=2,
+                               module=get_serialized_computation(
+                                   f_jax, x1,
+                                   abstracted_axes=({
+                                       0: 'b'
+                                   },)),
+                               Tout=[jax_res.dtype],
+                               Sout=[jax_res.shape],
+                               dim_args_spec=('0.0',))
+
+    res = tf.function(f_tf, jit_compile=True, autograph=False)(x1)
+    self.assertAllClose(
+        tf.nest.map_structure(lambda t: t.numpy(), res), jax_res)
+
 
 if __name__ == "__main__":
-  # TODO: Remove once tensorflow is 2.10.0 everywhere.
-  if not hasattr(tfxla, "optimization_barrier"):
-    jax.config.update("jax_remat_opt_barrier", False)
   absltest.main(testLoader=jtu.JaxTestLoader())
