@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Tests for the shape-polymorphic jax2tf conversion."""
+import unittest
 
 from absl.testing import absltest, parameterized
 from typing import Callable, Dict, List, Optional, Sequence, Tuple, Union
@@ -28,12 +29,11 @@ from jax import core
 from jax.experimental import jax2tf
 from jax.experimental.jax2tf import shape_poly
 from jax import lax
-from jax import linear_util as lu
 import jax.numpy as jnp
+from jax import random
 from jax._src import test_util as jtu
 from jax._src.lax import lax as lax_internal
 from jax._src.lax import control_flow as lax_control_flow
-from jax._src import util
 import numpy as np
 
 from jax.experimental.jax2tf.tests import tf_test_util
@@ -69,7 +69,7 @@ class DimPolynomialTest(tf_test_util.JaxToTfTestCase):
 
   a, b = shape_poly._parse_spec("a, b", (2, 3))
   @parameterized.named_parameters(
-      dict(testcase_name=f"_dim_spec={dim_spec}",
+      dict(testcase_name=f"_{dim_spec=}",
            dim_spec=dim_spec, dim_poly=dim_poly)
       for dim_spec, dim_poly in [
           ("2*a*b", 2 * a * b),
@@ -88,7 +88,7 @@ class DimPolynomialTest(tf_test_util.JaxToTfTestCase):
     self.assertEqual((dim_poly,), shape_poly._parse_spec(str(dim_poly), (2,)))
 
   @parameterized.named_parameters(
-      dict(testcase_name=f"_dim_spec={dim_spec}",
+      dict(testcase_name=f"_{dim_spec=}",
            dim_spec=dim_spec, dim_poly=dim_poly)
       for dim_spec, dim_poly in [
           ("2*a*b", 2 * a * b),
@@ -334,6 +334,7 @@ class DimPolynomialTest(tf_test_util.JaxToTfTestCase):
 
 class ShapePolyTest(tf_test_util.JaxToTfTestCase):
 
+
   def test_simple_unary(self):
     """Test shape polymorphism for a simple case, unary function."""
 
@@ -388,6 +389,20 @@ class ShapePolyTest(tf_test_util.JaxToTfTestCase):
         polymorphic_shapes=PS("h", "h"),
         expected_output_signature=tf.TensorSpec([None, None]))
 
+  def test_arange(self):
+    def f_jax(x):
+      return x + jnp.arange(x.shape[0], dtype=np.float32)
+    x = np.ones((3,), dtype=np.float32)
+    self.assertAllClose(jax2tf.convert(f_jax, polymorphic_shapes="b")(x),
+                        f_jax(x))
+
+  def test_argmax(self):
+    def f_jax(x):  # x: f32[b, 4, 5]
+      return lax.argmax(x, axis=1, index_dtype=np.int32)
+    x = np.arange(3 * 4 * 5, dtype=np.float32).reshape((3, 4, 5))
+    self.assertAllClose(jax2tf.convert(f_jax, polymorphic_shapes="(b, _, _)")(x),
+                        f_jax(x))
+
   def test_static_shape_result(self):
     """The result has static shape."""
 
@@ -697,14 +712,15 @@ class ShapePolyTest(tf_test_util.JaxToTfTestCase):
         expected_output_signature=tf.TensorSpec([4]))
 
   def test_with_nested_jit(self):
-    @jax.jit
-    def f_jax(x):
-      return jnp.sin(x)
-
+    x = np.ones((3, 4), dtype=np.float32)
+    # We implement the following computation
+    _ = x + (np.sin(x) + np.broadcast_to(np.arange(x.shape[1]), x.shape))
+    def f_jax(x):  # x: f32[w, h]
+      return jnp.sin(x) + jnp.arange(x.shape[1], dtype=x.dtype)
     self.CheckShapePolymorphism(
-        f_jax,
-        input_signature=[tf.TensorSpec([1, None])],
-        polymorphic_shapes=["1, b"])
+        lambda x: x + jax.jit(f_jax)(x),
+        input_signature=[tf.TensorSpec([None, None])],
+        polymorphic_shapes=["w, h"])
 
   def test_with_custom_vjp(self):
     """Shape-polymorphic custom VJP."""
@@ -834,6 +850,17 @@ class ShapePolyTest(tf_test_util.JaxToTfTestCase):
         res_jax,
         jax2tf.convert(f, polymorphic_shapes=["(b, h)", "h"])(x, y))
 
+  def test_while(self):
+    def f(x):
+      # x: f32[B], iter: i32
+      return lax.while_loop(lambda x_iter: x_iter[1] < 5,
+                            lambda x_iter: (x_iter[0] + jnp.arange(x_iter[0].shape[0], dtype=np.float32), x_iter[1] + 1),
+                            (x, 0))
+
+    x = np.ones((3,), dtype=np.float32)
+    res_tf = jax2tf.convert(f, polymorphic_shapes=["(b,)"])(x)
+    self.assertAllClose(f(x), res_tf)
+
   @jtu.sample_product(with_function=[False, True])
   def test_grad_int(self, with_function=True):
     # https://github.com/google/jax/issues/7093
@@ -866,6 +893,30 @@ class ShapePolyTest(tf_test_util.JaxToTfTestCase):
     self.assertAllClose(g_tf[0][1], (xi * 2).astype(yf.dtype))
     self.assertAllClose(g_tf[1], np.zeros_like(zb))
 
+  def test_prng(self):
+    # The PRNG implementation uses opaque types, test shape polymorphism
+    try:
+      prev_custom_prng = config.jax_enable_custom_prng
+      config.update("jax_enable_custom_prng", True)
+
+      def f_jax(x):  # x: f32[b1, b2]
+        key = random.PRNGKey(123)  #  key
+        # Exercise key operations that have custom lowering rules
+        broadcast_keys = lax.broadcast_in_dim(key, x.shape, ())  # key[b1, b2]
+        gather_keys = lax.broadcast_in_dim(broadcast_keys[0], (1, x.shape[1]), (1,))  # : key[1, b2]
+        slice_keys1 = lax.slice(broadcast_keys, (0, 0), (1, x.shape[1]), (1, 1))  # key[1, b2]
+        slice_keys2 = lax.dynamic_slice(broadcast_keys, (0, 0), slice_sizes=(1, x.shape[1]))  # key[1, b2]
+        upd1 = lax.dynamic_update_slice(slice_keys2, slice_keys1, start_indices=(0, 0))  # key[1, b2]
+        _ = lax.dynamic_update_slice(upd1, gather_keys, start_indices=(0, 0))
+        return x
+
+      self.CheckShapePolymorphism(f_jax,
+                                  input_signature=[tf.TensorSpec([None, None], dtype=tf.float32)],
+                                  polymorphic_shapes=["b1, b2"])
+    finally:
+      config.update("jax_enable_custom_prng", prev_custom_prng)
+
+
   def test_saved_model(self):
     f_jax = jnp.sin
     f_tf = jax2tf.convert(f_jax, polymorphic_shapes=["(b, ...)"])
@@ -1017,6 +1068,10 @@ class ShapePolyTest(tf_test_util.JaxToTfTestCase):
 
 class DimAsValueTest(tf_test_util.JaxToTfTestCase):
   """Dimension polynomials used as values in the JAX computation."""
+  def setUp(self):
+    super().setUp()
+    if config.jax2tf_default_experimental_native_lowering and not config.jax_dynamic_shapes:
+      self.skipTest("shape polymorphism but --jax_dynamic_shapes is not set.")
 
   def test_dynamic_shapes(self):
     # Test dim_as_value with dynamic shapes.
@@ -1220,7 +1275,7 @@ _POLY_SHAPE_TEST_HARNESSES = [
                   enable_and_disable_xla=True),
     [
         _make_harness("average",
-                      f"axis={axis}_weights=None",
+                      f"{axis=}_weights=None",
                       lambda x, axis: jnp.average(x, axis=axis, returned=False, weights=None),
                       [RandArg((7, 8, 4), _f32), StaticArg(axis)],
                       poly_axes=[0])
@@ -1228,7 +1283,7 @@ _POLY_SHAPE_TEST_HARNESSES = [
     ],
     [
         _make_harness("average",
-                      f"axis={axis}_weights=Some",
+                      f"{axis=}_weights=Some",
                       lambda x, weights, axis: jnp.average(x, axis=axis, returned=False, weights=weights),
                       [RandArg((7, 8, 4), _f32), RandArg((7, 8, 4), _f32), StaticArg(axis)],
                       poly_axes=[0, 0])
@@ -1556,7 +1611,7 @@ _POLY_SHAPE_TEST_HARNESSES = [
                   tol=1e-5),
     [
         _make_harness("mean",
-                      f"axis={axis}_keepdims={keepdims}_where=None",
+                      f"{axis=}_{keepdims=}_where=None",
                       lambda x, axis, keepdims: jnp.mean(x, axis=axis, keepdims=keepdims, where=None),
                       [RandArg((7, 8, 4), _f32), StaticArg(axis), StaticArg(keepdims)],
                       poly_axes=[0])
@@ -1565,7 +1620,7 @@ _POLY_SHAPE_TEST_HARNESSES = [
     ],
     [
         _make_harness("mean",
-                      f"axis={axis}_keepdims={keepdims}_where=Some",
+                      f"{axis=}_{keepdims=}_where=Some",
                       lambda x, where, axis, keepdims: jnp.mean(x, axis=axis, keepdims=keepdims, where=where),
                       [RandArg((7, 8, 4), _f32), RandArg((7, 8, 4), np.bool_), StaticArg(axis), StaticArg(keepdims)],
                       poly_axes=[0, 0])
@@ -1702,7 +1757,14 @@ _POLY_SHAPE_TEST_HARNESSES = [
                   expect_error=(core.InconclusiveDimensionOperation,
                                 re.escape(
                                   "Cannot divide evenly the sizes of shapes (b0, 2, 4) and (b0, -1, 3)"))),
-
+    _make_harness("roll", "axis=0",
+                  lambda x: jnp.roll(x, 2, axis=0),
+                  [RandArg((3, 4), _f32)],
+                  poly_axes=[0]),
+    _make_harness("roll", "axis=None",
+                  lambda x: jnp.roll(x, 2),
+                  [RandArg((3, 4), _f32)],
+                  poly_axes=[0]),
     _make_harness("scatter_add", "",
                   partial(lax.scatter_add, indices_are_sorted=False, unique_indices=True),
                   [RandArg((7, 4), _f32),
@@ -1802,7 +1864,7 @@ _POLY_SHAPE_TEST_HARNESSES = [
                   poly_axes=[0]),
     [
         _make_harness("var",
-                      f"axis={axis}_keepdims={keepdims}_where=None",
+                      f"{axis=}_{keepdims=}_where=None",
                       lambda x, axis, keepdims: jnp.var(x, axis=axis, keepdims=keepdims, where=None),
                       [RandArg((7, 8, 4), _f32), StaticArg(axis), StaticArg(keepdims)],
                       poly_axes=[0])
@@ -1811,7 +1873,7 @@ _POLY_SHAPE_TEST_HARNESSES = [
     ],
     [
         _make_harness("var",
-                      f"axis={axis}_keepdims={keepdims}_where=Some",
+                      f"{axis=}_{keepdims=}_where=Some",
                       lambda x, where, axis, keepdims: jnp.var(x, axis=axis, keepdims=keepdims, where=where),
                       [RandArg((7, 8, 4), _f32), RandArg((7, 8, 4), np.bool_), StaticArg(axis), StaticArg(keepdims)],
                       poly_axes=[0, 0])
@@ -1913,9 +1975,11 @@ class ShapePolyPrimitivesTest(tf_test_util.JaxToTfTestCase):
   # to parameterized below.
   @primitive_harness.parameterized(
       _flatten_harnesses(_POLY_SHAPE_TEST_HARNESSES),
-      #one_containing="repeat_repeats=poly_axis=None_scalar_poly_axes=[None, 0]",
+      #one_containing="roll_axis=None",
   )
   def test_prim(self, harness: Harness):
+    if config.jax2tf_default_experimental_native_lowering and not harness.params.get("enable_xla", True):
+      raise unittest.SkipTest("disabled for experimental_native_lowering and enable_xla=False")
     _test_one_harness(self, harness)
 
   def test_vmap_while(self):
@@ -2068,11 +2132,11 @@ class ShapePolyVmapPrimitivesTest(tf_test_util.JaxToTfTestCase):
 
   # For each primitive "xxx" the test will be called "test_vmap_prim_xxx_...".
   # If you want to run this test for only one harness that includes "foo"
-  # in the name (after test_prim), add parameter `one_containing="foo"`
+  # in the name (after test_vmap_prim), add parameter `one_containing="foo"`
   # to parameterized below.
   @primitive_harness.parameterized(
       _flatten_harnesses(_POLY_SHAPE_VMAP_TEST_HARNESSES),
-      #one_containing="eig_shape=float32[0,0]_computelefteigenvectors=False_computerighteigenvectors=False_poly_axes=[0]"
+      one_containing=""
   )
   def test_vmap_prim(self, harness: Harness):
     return _test_one_harness(self, harness)
