@@ -13,14 +13,16 @@
 # limitations under the License.
 
 """BCSR (Bached compressed row) matrix object and associated primitives."""
+from __future__ import annotations
 
 import operator
 
-from typing import NamedTuple, Sequence, Tuple
+from typing import NamedTuple, Optional, Sequence, Tuple
 
 import numpy as np
 
 from jax import core
+from jax import tree_util
 from jax.experimental.sparse._base import JAXSparse
 from jax.experimental.sparse import bcoo
 from jax.experimental.sparse.util import _broadcasting_vmap, _count_stored_elements, _csr_to_coo, _safe_asarray
@@ -28,6 +30,7 @@ import jax.numpy as jnp
 from jax.util import split_list, safe_zip
 from jax.interpreters import batching
 from jax.interpreters import mlir
+from jax._src.typing import Array, ArrayLike, DTypeLike
 
 Shape = Tuple[int, ...]
 
@@ -38,7 +41,7 @@ class BCSRProperties(NamedTuple):
   nse: int
 
 
-def _compatible(shape1, shape2):
+def _compatible(shape1: Sequence[int], shape2: Sequence[int]) -> bool:
   return all(s1 in (1, s2) for s1, s2 in safe_zip(shape1, shape2))
 
 
@@ -54,11 +57,9 @@ def _validate_bcsr_indices(indices: jnp.ndarray, indptr: jnp.ndarray,
   assert n_dense >= 0
 
   if not _compatible(indices.shape[:n_batch], shape[:n_batch]):
-    raise ValueError("indices batch dimensions not compatible for "
-                     f"indices.shape={indices.shape}, shape={shape}")
+    raise ValueError(f"indices batch dimensions not compatible for {indices.shape=}, {shape=}")
   if not _compatible(indptr.shape[:n_batch], shape[:n_batch]):
-    raise ValueError("indptr batch dimensions not compatible for "
-                     f"indptr.shape={indptr.shape}, shape={shape}")
+    raise ValueError(f"indptr batch dimensions not compatible for {indptr.shape=}, {shape=}")
   if indptr.shape[n_batch:] != (shape[n_batch] + 1,):
     raise ValueError("indptr shape must match the matrix shape plus 1.")
 
@@ -75,11 +76,9 @@ def _validate_bcsr(data: jnp.ndarray, indices: jnp.ndarray,
     raise ValueError("BCSR array must have 2 sparse dimensions; "
                      f"{n_sparse} is given.")
   if not _compatible(data.shape[:n_batch], shape[:n_batch]):
-    raise ValueError("data batch dimensions not compatible for "
-                    f"data.shape={data.shape}, shape={shape}")
+    raise ValueError(f"data batch dimensions not compatible for {data.shape=}, {shape=}")
   if data.shape[-(n_dense + 1):] != (nse,) + shape[n_batch + 2:]:
-    raise ValueError(f"Invalid data.shape={data.shape} for "
-                    f"nse={nse}, n_batch={n_batch}, n_dense={n_dense}")
+    raise ValueError(f"Invalid {data.shape=} for {nse=}, {n_batch=}, {n_dense=}")
   return props
 
 
@@ -106,7 +105,8 @@ value to the nse (number of stored elements) argument.
 """
 
 
-def bcsr_fromdense(mat, *, nse=None, n_batch=0, n_dense=0, index_dtype=jnp.int32):
+def bcsr_fromdense(mat: ArrayLike, *, nse: Optional[int] = None, n_batch: int = 0,
+                   n_dense:int = 0, index_dtype: DTypeLike = jnp.int32) -> BCSR:
   """Create BCSR-format sparse matrix from a dense matrix.
 
   Args:
@@ -122,13 +122,14 @@ def bcsr_fromdense(mat, *, nse=None, n_batch=0, n_dense=0, index_dtype=jnp.int32
   mat = jnp.asarray(mat)
   if nse is None:
     nse = _count_stored_elements(mat, n_batch, n_dense)
-  nse = core.concrete_or_error(operator.index, nse, _TRACED_NSE_ERROR)
-  return BCSR(_bcsr_fromdense(mat, nse=nse, n_batch=n_batch, n_dense=n_dense,
+  nse_int: int = core.concrete_or_error(operator.index, nse, _TRACED_NSE_ERROR)
+  return BCSR(_bcsr_fromdense(mat, nse=nse_int, n_batch=n_batch, n_dense=n_dense,
                               index_dtype=index_dtype),
               shape=mat.shape)
 
 
-def _bcsr_fromdense(mat, *, nse, n_batch=0, n_dense=0, index_dtype=jnp.int32):
+def _bcsr_fromdense(mat: ArrayLike, *, nse: int, n_batch: int = 0, n_dense: int = 0,
+                    index_dtype: DTypeLike = jnp.int32) -> Tuple[Array, Array, Array]:
   """Create BCSR-format sparse matrix from a dense matrix.
 
   Args:
@@ -183,7 +184,7 @@ def _bcsr_fromdense_batching_rule(batched_args, batch_dims, *, nse, n_batch,
                                   n_dense, index_dtype):
   M, = batched_args
   if batch_dims != (0,):
-    raise NotImplementedError(f"batch_dims={batch_dims}")
+    raise NotImplementedError(f"{batch_dims=}")
   new_n_batch = n_batch + 1
   n_sparse = M.ndim - n_dense - new_n_batch
   if n_sparse != 2:
@@ -203,7 +204,7 @@ mlir.register_lowering(bcsr_fromdense_p, mlir.lower_fun(
 bcsr_todense_p = core.Primitive('bcsr_todense')
 
 
-def bcsr_todense(mat):
+def bcsr_todense(mat: BCSR) -> Array:
   """Convert batched sparse matrix to a dense matrix.
 
   Args:
@@ -216,7 +217,7 @@ def bcsr_todense(mat):
                        shape=tuple(mat.shape))
 
 
-def _bcsr_todense(data, indices, indptr, *, shape):
+def _bcsr_todense(data: ArrayLike, indices: ArrayLike, indptr: ArrayLike, *, shape: Shape) -> Array:
   """Convert batched sparse matrix to a dense matrix.
 
   Args:
@@ -247,8 +248,7 @@ def _bcsr_todense_abstract_eval(data, indices, indptr, *, shape):
 def _bcsr_todense_batching_rule(batched_args, batch_dims, *, shape):
   data, indices, indptr = batched_args
   if any(b not in [0, None] for b in batch_dims):
-    raise NotImplementedError(f"batch_dims={batch_dims}. "
-                              "Only 0 and None are supported.")
+    raise NotImplementedError(f"{batch_dims=}. Only 0 and None are supported.")
   if batch_dims[0] is None:
     data = data[None, ...]
   if batch_dims[1] is None:
@@ -267,7 +267,7 @@ mlir.register_lowering(bcsr_todense_p, mlir.lower_fun(
 bcsr_extract_p = core.Primitive('bcsr_extract')
 
 
-def bcsr_extract(indices, indptr, mat):
+def bcsr_extract(indices: ArrayLike, indptr: ArrayLike, mat: ArrayLike) -> Array:
   """Extract values from a dense matrix at given BCSR (indices, indptr).
 
   Args:
@@ -299,6 +299,7 @@ mlir.register_lowering(bcsr_extract_p, mlir.lower_fun(
     _bcsr_extract_impl, multiple_results=False))
 
 
+@tree_util.register_pytree_node_class
 class BCSR(JAXSparse):
   """Experimental batched CSR matrix implemented in JAX."""
 
@@ -333,9 +334,9 @@ class BCSR(JAXSparse):
     except Exception:  # pylint: disable=broad-except
       repr_ = f"{name}(<invalid>)"
     else:
-      extra = f", nse={nse}"
-      if n_batch: extra += f", n_batch={n_batch}"
-      if n_dense: extra += f", n_dense={n_dense}"
+      extra = f", {nse=}"
+      if n_batch: extra += f", {n_batch=}"
+      if n_dense: extra += f", {n_dense=}"
       repr_ = f"{name}({dtype}{shape}{extra})"
     if isinstance(self.data, core.Tracer):
       repr_ = f"{type(self.data).__name__}[{repr_}]"
@@ -345,7 +346,7 @@ class BCSR(JAXSparse):
     raise NotImplementedError("Tranpose is not implemented.")
 
   def tree_flatten(self):
-    return (self.data, self.indices, self.indptr), {}
+    return (self.data, self.indices, self.indptr), {'shape': self.shape}
 
   @classmethod
   def _empty(cls, shape, *, dtype=None, index_dtype='int32', n_dense=0,
@@ -353,8 +354,7 @@ class BCSR(JAXSparse):
     """Create an empty BCSR instance. Public method is sparse.empty()."""
     shape = tuple(shape)
     if n_dense < 0 or n_batch < 0 or nse < 0:
-      raise ValueError(f"Invalid inputs: shape={shape}, n_dense={n_dense},"
-                       f"n_batch={n_batch}, nse={nse}")
+      raise ValueError(f"Invalid inputs: {shape=}, {n_dense=}, {n_batch=}, {nse=}")
     n_sparse = len(shape) - n_dense - n_batch
     if n_sparse != 2:
       raise ValueError("BCSR sparse.empty: must have 2 sparse dimensions.")
