@@ -15,6 +15,7 @@
 """Sparse utilities."""
 
 import functools
+from typing import Any, Iterable, Sequence, Tuple, Union
 
 import numpy as np
 import jax
@@ -27,6 +28,8 @@ from jax._src import stages
 from jax._src.api_util import flatten_axes
 import jax.numpy as jnp
 from jax.util import safe_zip
+from jax._src.lax.lax import _dot_general_shape_rule, DotDimensionNumbers
+from jax._src.typing import Array
 
 class SparseEfficiencyError(ValueError):
   pass
@@ -42,7 +45,7 @@ class CuSparseEfficiencyWarning(SparseEfficiencyWarning):
 # TODO: possibly make these primitives, targeting cusparse rountines
 #       csr2coo/coo2csr/SPDDMM
 
-def _asarray_or_float0(arg):
+def _asarray_or_float0(arg) -> Union[np.ndarray, Array]:
   if isinstance(arg, np.ndarray) and arg.dtype == dtypes.float0:
     return arg
   return jnp.asarray(arg)
@@ -67,43 +70,54 @@ def _broadcasting_vmap(fun, in_axes=0, out_axes=0):
   return batched_fun
 
 @jax.jit
-def _csr_to_coo(indices, indptr):
+def _csr_to_coo(indices: Array, indptr: Array) -> Tuple[Array, Array]:
   """Given CSR (indices, indptr) return COO (row, col)"""
   return jnp.cumsum(jnp.zeros_like(indices).at[indptr].add(1)) - 1, indices
 
-def _csr_extract(indices, indptr, mat):
+def _csr_extract(indices: Array, indptr: Array, mat: Array) -> Array:
   """Extract values of dense matrix mat at given CSR indices."""
-  return _coo_extract(*_csr_to_coo(indices, indptr), mat)
+  row, col = _csr_to_coo(indices, indptr)
+  return _coo_extract(row, col, mat)
 
-def _coo_extract(row, col, mat):
+def _coo_extract(row: Array, col: Array, mat: Array) -> Array:
   """Extract values of dense matrix mat at given COO indices."""
   return mat[row, col]
 
-def _count_stored_elements_per_batch(mat, n_batch=0, n_dense=0):
+def _count_stored_elements_per_batch(mat: Array, n_batch: int = 0, n_dense: int = 0) -> Array:
   """Return per-batch number of stored elements (nse) of a dense matrix."""
   mat = jnp.asarray(mat)
   mask = (mat != 0)
   if n_dense > 0:
-    mask = mask.any([-(i + 1) for i in range(n_dense)])
-  mask = mask.sum(list(range(n_batch, mask.ndim)))
+    mask = mask.any(tuple(-(i + 1) for i in range(n_dense)))
+  mask = mask.sum(tuple(range(n_batch, mask.ndim)))
   return mask
 
-def _count_stored_elements(mat, n_batch=0, n_dense=0):
+def _count_stored_elements(mat: Array, n_batch: int = 0, n_dense: int = 0) -> int:
   """Return the number of stored elements (nse) of the given dense matrix."""
   return int(_count_stored_elements_per_batch(mat, n_batch, n_dense).max())
 
-def _is_pytree_placeholder(*args):
+def _is_pytree_placeholder(*args: Any) -> bool:
   # Returns True if the arguments are consistent with being a placeholder within
   # pytree validation.
   return all(type(arg) is object for arg in args) or all(arg is None for arg in args)
 
-def _is_aval(*args):
+def _is_aval(*args: Any) -> bool:
   return all(isinstance(arg, core.AbstractValue) for arg in args)
 
-def _is_arginfo(*args):
+def _is_arginfo(*args: Any) -> bool:
   return all(isinstance(arg, stages.ArgInfo) for arg in args)
 
-def _safe_asarray(args):
+def _safe_asarray(args: Sequence[Any]) -> Iterable[Union[np.ndarray, Array]]:
   if _is_pytree_placeholder(*args) or _is_aval(*args) or _is_arginfo(*args):
     return args
   return map(_asarray_or_float0, args)
+
+def _dot_general_validated_shape(
+    lhs_shape: Tuple[int, ...], rhs_shape: Tuple[int, ...],
+    dimension_numbers: DotDimensionNumbers) -> Tuple[int, ...]:
+  """Validate the inputs and return the output shape."""
+  lhs = core.ShapedArray(lhs_shape, np.float32)
+  rhs = core.ShapedArray(rhs_shape, np.float32)
+  return _dot_general_shape_rule(
+    lhs, rhs, dimension_numbers=dimension_numbers,
+    precision=None, preferred_element_type=None)
