@@ -357,7 +357,6 @@ class BatchTrace(Trace):
     src = source_info_util.current()
     if primitive.multiple_results:
       return [BatchTracer(self, x, d, src) for x, d in zip(val_out, dim_out)]
-      return map(partial(BatchTracer, self), val_out, dim_out)
     else:
       return BatchTracer(self, val_out, dim_out, src)
 
@@ -878,8 +877,7 @@ def matchaxis(axis_name, sz, src, dst, x, sum_match=False):
   else:
     if (not isinstance(axis_name, core._TempAxisName) and
         axis_name is not core.no_axis_name):
-      raise ValueError(f'vmap has mapped output (axis_name={axis_name}) '
-                       f'but out_axes is {dst}')
+      raise ValueError(f'vmap has mapped output ({axis_name=}) but out_axes is {dst}')
     else:
       raise ValueError(f'vmap has mapped output but out_axes is {dst}')
 
