@@ -191,8 +191,10 @@ from jax._src.numpy.lax_numpy import (
     percentile as percentile,
     pi as pi,
     piecewise as piecewise,
+    place as place,
     printoptions as printoptions,
     promote_types as promote_types,
+    put as put,
     quantile as quantile,
     ravel as ravel,
     ravel_multi_index as ravel_multi_index,
